#!/bin/bash -e
#
# install-api:install-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo >&2 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.'
  exit 100
fi
if [ "$( type -t exit_if_variable_not_define_or_empty )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required Function exit_if_variable_not_define_or_empty not found.'
  exit 100
fi
if [ "$( type -t install_api_install_default_desktop_icons )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/install-desktop-api.sh" || exit $?
fi

declare -grx INSTALL_API_DEFAULT_GLOBAL_DESKTOP_DIR='/usr/local/share/applications'

function _get_package_manager_ {
  local pkg_manager="$1" # R/W

  if [ -z "${pkg_manager}" ] ; then
    pkg_manager="$( command -v apt-get || command -v pkg || command -v yum )"
  fi

  if [ -z "${pkg_manager}" ] ; then
    echofatal 'Package manager not found.'
    return 100
  fi
  echo "${pkg_manager}"
}

function install_update {
  local pkg_manager=
  pkg_manager="$( _get_package_manager_ "$1" )"
  if [ -z "${pkg_manager}" ] ; then
    return 101
  fi

  "${pkg_manager}" update || return $?
}

function install_install_if_needed {
  local pkg_manager=
  pkg_manager="$( _get_package_manager_ "$1" )"
  if [ -z "${pkg_manager}" ] ; then
    return 101
  fi
  shift

  local command_name=
  local package_name=

  while [ "$#" -gt 0 ] ; do
    IFS=";" read -r command_name package_name <<< "${1}"
    shift

    if [ -z "${package_name}" ] ; then
      package_name="${command_name}"
    fi

    if ! command -v "${command_name}" >/dev/null ; then
      "${pkg_manager}" install -y "${package_name}" || return $?
    fi
  done
}

function install_if_needed { # DEPRECATED 2022-00-00 # REMOVED 2023-04-02
  echofatal 'install_if_needed is deprecated, use install_update and install_install_if_needed instead'
  exit 123
}

function install_set_chmod_for_list {
  local -r mode="$1"
  local -r files="$2"

  local file=
  while read -r file ; do
    if [ -n "${file}" ] ; then
      chmod -v "${mode}" "${INSTALL_DATA_DIR}/${file}" || return $?
    fi
  done <<<"${files}"
}

function install_links {
  local -r files="$1"

  local file=
  local realfile=
  local linkfile=
  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ ! -f "${realfile}" ] ; then
        echofatal "'${realfile}' not found."
        return 101
      fi

      if [ -L "${linkfile}" ] ; then
        if [ "$( realpath "${linkfile}" )" = "${realfile}" ] ; then
          rm -v "${linkfile}"
        fi
      fi
      ln -sv "${realfile}" "${linkfile}" || return $?
    fi
  done <<<"${files}"
}

function install_main {
  mkdir -pv "${INSTALL_DATA_DIR}" || return $?

  local -a rsync_params=(-av)

  if [ -z "${INSTALL_API_INSTALL_MAIN_EXCLUDE:-}" ] ; then
    echowarn 'INSTALL_API_INSTALL_MAIN_EXCLUDE not defined, use defaults.'
    rsync_params+=(--exclude='.git')
    rsync_params+=(--exclude='.gitignore')
    rsync_params+=(--exclude='.gitlab-ci.yml')
    rsync_params+=(--exclude='package') # FIXME: should be removed
    rsync_params+=(--exclude='samples') # FIXME: should be removed
  else
    for exclude in "${INSTALL_API_INSTALL_MAIN_EXCLUDE[@]}" ] ; do
      rsync_params+=(--exclude="${exclude}")
    done
  fi
  rsync "${rsync_params[@]}" "${PROJECT_HOME}/"* "${INSTALL_DATA_DIR}" || return $?
  chown -Rv root:root "${INSTALL_DATA_DIR}" || return $?

  find "${INSTALL_DATA_DIR}" -type f -exec chmod -v 444 {} \; || return $?
  find "${INSTALL_DATA_DIR}" -type d -exec chmod -v 755 {} \; || return $?

  install_set_chmod_for_list 555 "${INSTALL_EXE}" || return $?
  install_links "${INSTALL_LINKS}" || return $?
  install_api_install_default_desktop_icons || return $?

  echoinfo "$( realpath "$0" ) done."
}

function exit_if_not_root {
  if [ "${EUID}" -ne 0 ] ; then
    echofatal 'This script must be run as root.'
    exit 100
  fi
}

function remove_previous_version_before_install {
  local remove='false'

  if [ -f "${REMOVE_EXE}" ] ; then
    echoinfo "Using: '${REMOVE_EXE}' --force-remove"
    bash "${REMOVE_EXE}" --force-remove || true
    remove='true'
  elif [ -n "${REMOVE_OLD:-}" ] && [ -f "${REMOVE_OLD}" ] ; then
    echoinfo "Using: '${REMOVE_OLD}' --force-remove"
    bash "${REMOVE_OLD}" --force-remove  || true
    remove='true'
  fi

  if [ "${remove}" = 'false' ] ; then
    echowarn "File not found: '${REMOVE_EXE}'"
  fi
}

function remove_links {
  local -r files="$1"

  local file=
  local realfile=
  local linkfile=
  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ -L "${linkfile}" ] ; then
        if [ "$( readlink "${linkfile}" )" = "${realfile}" ] || [ -f "$( readlink "${linkfile}" )" ] ; then
          rm -v "${linkfile}" || true # Ignore errors
        fi
      fi
    fi
  done <<<"${files}"
}

function _remove_main_ {
  local -r force_remove="${0:-false}"

  exit_if_not_root

  local scriptpath=
  scriptpath="$( realpath "$0" )" || return $?

  local remove='false'
  if [ "${force_remove}" = 'false' ] ; then
    if [ "${REMOVE_EXE}" != "${scriptpath}" ] &&  [ -f "${REMOVE_EXE}" ] ; then
      echoinfo "Using: '${REMOVE_EXE}' --force-remove"
      bash "${REMOVE_EXE}" --force-remove
      remove='true'
    elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
      if [ -f "${REMOVE_OLD}" ] ; then
        echoinfo "Using: '${REMOVE_OLD}' --force-remove"
        bash "${REMOVE_OLD}" --force-remove
        remove='true'
      fi
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    install_api_remove_default_desktop_icons || true # Ignore errors
    remove_links "${INSTALL_LINKS}" || true # Ignore errors

    rm -fvr "${INSTALL_DATA_DIR}" || return $?
  fi

  echoinfo "${scriptpath} done."
}

function _remove_main_usage_ {
  cat <<EOF
Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) [--force-remove]

Options:
  -h,       --help  Display help message and exit.
            --force-remove
                    Use this script to remove, don't try to find previous version.

EOF
}

function remove_main {
  local param=
  local param_force_remove='false'

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        _remove_main_usage_
        exit 0 # exit caller
        ;;

      '--force-remove')
        param_force_remove='true'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        _remove_main_usage_ >&2
        return 1
        ;;
    esac
  done

  _remove_main_ "${param_force_remove}" || return $?
}


exit_if_variable_not_define_or_empty 'INSTALL_BIN_DIR'
exit_if_variable_not_define_or_empty 'INSTALL_DATA_DIR'
exit_if_variable_not_define_or_empty 'REMOVE_EXE'
