#!/bin/bash
#
# install-api:make-package-api.sh
#

if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t P_param )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function P_param not found.'
  exit 1
fi
if ! is_variable_define_and_not_empty_safe TARGET_HOME ; then
  echofatal 'TARGET_HOME must be define.'
  exit 1
fi
if ! is_variable_define_and_not_empty_safe DIRECTORIES_TO_REMOVE_FROM_RELEASE ; then
  echowarn 'DIRECTORIES_TO_REMOVE_FROM_RELEASE is not define.'
  declare -gra DIRECTORIES_TO_REMOVE_FROM_RELEASE=()
fi

function test_tag {
  local -r tag="$1"

  if ! git tag | grep -q "^${tag}$" ; then
    echofatal "Tag '${tag}' not found."
    git tag >&2
    exit 1
  fi
}

function get_remote_name {
  git remote show | head -n 1
}

function get_remote_url {
  git remote get-url "$( get_remote_name )"
}

function __checkout_and_prepare_release__ {
  local -r remote_url="${1}"
  local -r tag_name="${2}"
  local    release_directory=

  release_directory="$( basename "${remote_url%.git}" )" || return $?
  local -r release_directory

  [ -d "${TARGET_HOME}" ] && rm -fr "${TARGET_HOME}"
  mkdir -v "${TARGET_HOME}" || return $?
  pushd "${TARGET_HOME}" >/dev/null || return $?

  git clone --recurse-submodules --branch "${tag_name}" "${remote_url}" "${release_directory}" || return $?

  pushd "${release_directory}" >/dev/null || return $?

  git checkout -b "package-${tag_name}" || return $?

  echoinfo "Prepare package into: '${TARGET_HOME}'"
  rm -fr '.git/'
  rm '.gitignore' || true
  rm '.gitlab-ci.yml' || true
  rm '.gitmodules'

  if [ "${#DIRECTORIES_TO_REMOVE_FROM_RELEASE[@]}" -gt 0 ] ; then
    local directory_to_remove=

    for directory_to_remove in "${DIRECTORIES_TO_REMOVE_FROM_RELEASE[@]}" ; do
      echoinfo "Removing directory: '${directory_to_remove}'"
      rm -fr "${directory_to_remove}"
    done
  fi

  popd >/dev/null || return $?

  local -r release_basename="${tag_name}"

  tar -czvf "${release_basename}.tar.gz" "${release_directory}" || return $?
  echoinfo "Now you need to upload: '${release_basename}.tar.gz'"
  rm -fr "${release_directory}" || return $?

  popd >/dev/null || return $?
}

function __do_prepare_release__ {
  local -r tag_name="$1"

  git fetch --verbose --prune || return $?

  test_tag "${tag_name}" || return $?

  local url=
  url="$( get_remote_url )" || return $?
  if [ -z "${url}" ] ; then
    echofatal "Can't found git remote URL."
    exit 1
  fi

  __checkout_and_prepare_release__ "${url}" "${tag_name}" || return $?
}


function make_usage {
  cat <<EOF
Checkout tag and create archive for release.

Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) --tag TAGNAME

Options:
  -h,           --help  Display help message and exit.
  -T TAGNAME,   --tag TAGNAME
                        Prepare release for TAGNAME
  -L,           --list  List tags

EOF
}

function make_main {
  local param=
  local param_tag_name=

  pushd "${PROJECT_HOME}" >/dev/null || return $?

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help') make_usage ; return 0 ;;
      '-T'|'--tag') param_tag_name="$( P_param "$@" )" && shift || return $? ;;

      '-L'|'--list')
        git fetch --all --quiet
        git tag || return 1
        return 0
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        make_usage >&2
        return 1
        ;;
    esac
  done

  if [ -z "${param_tag_name}" ] ; then
    echofatal "--tag value is missing."
    make_usage >&2
    return 1
  fi

  __do_prepare_release__ "${param_tag_name}" || return $?

  popd >/dev/null || return $?
  echoinfo "$( realpath "$0" ) done."
}
