#!/bin/bash
#
# ssh-api:ssh-common-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source message-api.sh" >&2
  exit 100
fi
if [ "$( type -t is_variable_define_and_not_empty_safe )" != 'function' ] ; then
  echofatal "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required."
  echofatal "ERROR in '${BASH_SOURCE[0]}' - source bash-common-api.sh"
  exit 100
fi
if ! command -v host >/dev/null ; then
  echofatal "ERROR in '${BASH_SOURCE[0]}' - host command is required."
  echofatal "ERROR in '${BASH_SOURCE[0]}' - sudo apt install dnsutils"
  exit 100
fi

# Try to use resolved IP instead of HOSTNAME
function __ssh_api_common_private_get_ssh_host_ip__ { # was: _ssh_api_common_get_ssh_host_ip_
  local -r _ssh_host_="$1"

  local _ssh_ip_host_=
  if [ "$( getRSYNC_RESOLV_HOST )" != 'false' ] ; then

    if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
      echodebug "${FUNCNAME[0]}() - $( host "${_ssh_host_}" )"
      echodebug "${FUNCNAME[0]}() - $( host "${_ssh_host_}" | sed -E 's,.* (.*)$,\1,g' )"
    fi

    local _host_unsafe_result_=
    if _host_unsafe_result_="$( host "${_ssh_host_}" | sed -E 's,.* (.*)$,\1,g' )" ; then
      _ssh_ip_host_="${_host_unsafe_result_}"

      if [ "${_ssh_ip_host_: -1}" = '.' ] ; then
        # Last character of ${_ssh_ip_host_} is '.'
        _ssh_ip_host_="${_ssh_host_}" # result is a domain, assume ${_ssh_host_} is already an IP
      fi
    fi
  fi

  if [ -n "${_ssh_ip_host_}" ] ; then
    echo "${_ssh_ip_host_}"
  else
    echo "${_ssh_host_}"
  fi
}

function __ssh_api_common_private_get_user_from_hostname__ { # was: _ssh_api_get_user_from_hostname
  local -r hostname="$1"

  local user_varname=
  user_varname="$(
    tr '.' '_' <<<"${hostname}" |
    tr '-' '_' |
    tr '[:lower:]' '[:upper:]'
    )_SSH_USER"
  local -r user_varname

  if is_variable_define_and_not_empty_safe "${user_varname}" ; then
   echo "${!user_varname}"
  else
    echowarn "${FUNCNAME[0]}]( '${hostname}' ) - '${user_varname}' is not define"
    return 100
  fi
}

function getRSYNC_RESOLV_HOST {
  if [ -n "${RSYNC_RESOLV_HOST:-}" ] ; then
    # ${RSYNC_RESOLV_HOST} is define and not empty
    echo "${RSYNC_RESOLV_HOST}"
  else
    echo 'true'
  fi
}

function ssh_api_print_env {
  local -ri __line_max__=150

  env --null | sort -z | while IFS= read -r -d $'\0' __line__ ; do
    fixed_line="$(
      sed -z 's/\n/,/g;s/,$/\n/' <<<"${__line__}" |
        head -c "${__line_max__}"
        )"

    # printf crops the trailing newline, then add one to be sure to have only one newline
    printf '%s\n' "${fixed_line}"
  done
}

function ssh_api_is_SSH_API_SSH_DEBUG {
  if [ "${SSH_API_SSH_DEBUG:-}" = 'true' ] ; then
    return 0 # 'true'
  else
    return 1 # 'false'
  fi
}

function ssh_api_initialize_SSH_USER_or_exit {
  local -r _ssh_host_="$1"

  SSH_USER="$( __ssh_api_common_private_get_user_from_hostname__ "${_ssh_host_}" )" || return $?
  declare -gr SSH_USER

  if [ -z "${SSH_USER}" ] ; then
    echofatal "SSH_USER is not define (exit)"
    exit 1
  fi
}

function ssh_api_update_known_hosts {
  local -r host="$1"
  local -r key_type="${2:-}" # default ecdsa

  # Fix bad known_hosts files
  sed -i '/^null$/d' ~/.ssh/known_hosts || return $?

  local -a ssh_keyscan_params=()
  case "${key_type}" in
    '')
      echowarn "${FUNCNAME[0]}( '${host}' '${key_type}' ) - No type"
      ssh_keyscan_params+=(-t ecdsa) # Key ECDSA
      ;;

    'all')
      ssh_keyscan_params+=(-H) # All keys
      ;;

    'ecdsa'|'rsa')
      ssh_keyscan_params+=("${key_type}")
      ;;

    *)
      echowarn "${FUNCNAME[0]}( '${host}' '${key_type}' ) - Strange type"
      ssh_keyscan_params+=("${key_type}")
      ;;
  esac

  while ssh-keygen -f ~/.ssh/known_hosts -H -F "${host}" >/dev/null ; do
    ssh-keygen -f ~/.ssh/known_hosts -R "${host}"
  done
  result="$( ssh-keyscan "${ssh_keyscan_params[@]}" "${host}" )" || return $?
  echo "${result}" >> ~/.ssh/known_hosts
}

function ssh_api_start_ssh_agent {
  echoinfo "${FUNCNAME[0]}() - SSH_AUTH_SOCK='${SSH_AUTH_SOCK:-}' - SSH_AGENT_PID='${SSH_AGENT_PID:-}'"
  if [ -z "${SSH_AUTH_SOCK:-}" ] || [ -z "${SSH_AGENT_PID:-}" ] ; then
    echoinfo "${FUNCNAME[0]}() - start ssh-agent (ssh-agent -s)"

    # shellcheck disable=SC2046 # Quote this to prevent word splitting.
    eval $( ssh-agent -s ) >&2
    # Now SSH_AUTH_SOCK and SSH_AGENT_PID defined (and exported)
    echoinfo "${FUNCNAME[0]}() - SSH_AUTH_SOCK='${SSH_AUTH_SOCK:-}'"
    echoinfo "${FUNCNAME[0]}() - SSH_AGENT_PID='${SSH_AGENT_PID:-}'"

    if [ "${SSH_API_DISABLED_SSH_AGENT_ADD_PRIVATE_KEY:-}" != 'true' ] ; then
      # adds private key identities
      echoinfo "${FUNCNAME[0]}() - Add default keys (ssh-add)"
      if [ "$( ssh -V 2>&1 | sed -E -e 's,^OpenSSH_([0-9]+)\.(.*)$,\1,g'  )" -ge 8 ] ; then
        ssh-add -v >&2 # Require at least OpenSSH 8
      else
        ssh-add >&2
      fi
    fi
  fi
}

if [ "${SSH_API_DISABLED_SSH_AGENT_AUTO_START:-}" != 'true' ] ; then
  ssh_api_start_ssh_agent
fi
