#!/bin/bash -e
#
# rsync-now:bin/apis/rsync-now-api.sh
#

declare -gA gconfig_json_part=()
declare -ga gconfig_all_ids=()
declare -ga gconfig_choices=()
declare -ga gconfig_enabled_ids=()

function _build_gconfig_for_entry_ {
  local json="$1"

  local choice_id=
  choice_id="$( get_substituted_value_from_json '.local' "${json}" )" || return $?

  local choice_description=
  choice_description="$( get_substituted_value_from_json '.description' "${json}" )" || return $?

  local choice_enabled=
  choice_enabled="$( get_value_from_json '.enabled' "${json}" )" || return $?

  gconfig_choices+=( "${choice_id}" "${choice_description}" "${choice_enabled}")
  gconfig_json_part["${choice_id}"]="${json}"

  gconfig_all_ids+=( "${choice_id}" )
  if [ "${choice_enabled}" = 'on' ] ; then
    gconfig_enabled_ids+=( "${choice_id}" )
  fi
}

function build_gconfig {
  local -r json_config="$1"

  local json=
  while read -r json ; do
    if [ -n "${json}" ] ; then
      _build_gconfig_for_entry_ "${json}" || return $?
    fi
  done < <( get_rsync_configs "${json_config}" )
}

function rsync_now_display_choices {
  local -r _ssh_user_="$1"
  local -r _ssh_address_="$2"

  #
  # Display choices to user
  #
  local -r title='Backup'

  local result=
  local -i rc=0
  result="$( whiptail --separate-output --checklist "${title}" 15 80 5 "${gconfig_choices[@]}" 3>&1 1>&2 2>&3 )" || rc=$?

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "${FUNCNAME[0]}() - whiptail: rc=${rc}"
    echodebug "${FUNCNAME[0]}() - whiptail: result='result'"
  fi

  if [ "${rc}" -eq 0 ] ; then
    local -a selected_ids=()

    # Prepare array of selected_ids
    local selected_id=
    while IFS= read -r selected_id ; do
      if [ -n "${selected_id}" ] ; then
        selected_ids+=("${selected_id}")
      fi
    done <<<"${result}"

    rsync_now_for_ids "${_ssh_user_}" "${_ssh_address_}" "${selected_ids[@]}" || return $?
  else
    echo "Cancel by user." >&2
    exit 101
  fi
}

function rsync_now_for_all_ids {
  local -r _ssh_user_="$1"
  local -r _ssh_address_="$2"

  rsync_now_for_ids "${_ssh_user_}" "${_ssh_address_}" "${gconfig_all_ids[*]}" || return $?
}

function rsync_now_for_enabled_ids {
  local -r _ssh_user_="$1"
  local -r _ssh_address_="$2"

  rsync_now_for_ids "${_ssh_user_}" "${_ssh_address_}" "${gconfig_enabled_ids[*]}" || return $?
}

function _rsync_now_for_id_ {
  local -r _ssh_user_="$1"
  local -r _ssh_address_="$2"
  local -r _selected_id_="$3"
  local -r _json_="$4"

  local _local_directory_=
  _local_directory_="$( get_substituted_value_from_json '.local' "${_json_}" )" || return $?

  local _remote_directory_=
  _remote_directory_="$( get_substituted_value_from_json '.remote' "${_json_}" )" || return $?

  local _rsync_options_json_=
  _rsync_options_json_="$( get_substituted_options_json_config "${_json_}" )" || return $?

  echookay "SYNC: ${_local_directory_} -> ${_remote_directory_} (${_rsync_options_json_})"

  if [ -z "${_local_directory_}" ] ; then
    echoerror "No local directory defined for '${_selected_id_}'"
    return 100
  fi
  if [ ! -d "${_local_directory_}" ] ; then
    echoerror "Local directory not found for '${_selected_id_}': '${_local_directory_}'"
    return 100
  fi
  if [ -z "${_remote_directory_}" ] ; then
    echoerror "No remote directory defined for '${_selected_id_}'"
    return 100
  fi

  run_rsync_now_from_config "${_ssh_user_}" "${_ssh_address_}" \
    "${_local_directory_}" "${_remote_directory_}" "${_rsync_options_json_}" || return $?
}

function rsync_now_for_ids {
  local -r _ssh_user_="$1"
  local -r _ssh_address_="$2"
  shift 2
  # shellcheck disable=SC2206 # Quote to prevent word splitting/globbing, or split robustly with mapfile or read -a.
  local -a selected_ids=($@)

  local -i index=0
  local selected_id=
  for index in "${!selected_ids[@]}"; do
    selected_id="${selected_ids[${index}]}"
    echoinfo "Handle: ${selected_id}"

    _rsync_now_for_id_ "${_ssh_user_}" "${_ssh_address_}" "${selected_id}" "${gconfig_json_part[${selected_id}]}" || return $?
  done

  echodebug "selected_ids=${selected_ids[*]} - index=${index}/${#selected_ids[@]}"
  echodebug "Last selected_id=${selected_id}"
}

function run_rsync_now_from_config {
  local -r _ssh_user_="$1"
  local -r _ssh_address_="$2"
  local -r source_folder="$3"
  local -r target_folder="$4"
  local -r rsync_extra_options_json="$5"

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "rsync_extra_options_json: '$( jq '.' <<<"${rsync_extra_options_json}" )'"
    echodebug "type: '$( jq 'type' <<<"${rsync_extra_options_json}" )'"
  fi


  local options=
  options="$( get_options_for_rsync "${rsync_extra_options_json}" )" || return $?
  local -t options

  local -a xoptions=()
  if [ -n "${options}" ] ; then
    local _opt_=
    while read -r _opt_ ; do
      xoptions+=("${_opt_}" )
    done <<<"${options}"
  fi

  declare -gx RSYNC_NOW_NO_DEFAULT=
  get_options_nodefault_from_json "${rsync_extra_options_json}" || return $? # Set RSYNC_NOW_NO_DEFAULT

  if [ "${RSYNC_NOW_API_CHECKSUM:-}" = 'true' ] ; then
    xoptions+=(--checksum )
  fi
  echodebug "RSYNC_NOW_NO_DEFAULT: '${RSYNC_NOW_NO_DEFAULT}'"

  ssh_api_RSYNC_OPTS_ARRAY_set_opts # Init options (use system default)

  time {
    if [ "${RSYNC_NOW_NO_DEFAULT}" = 'true' ] ; then
      echookay 'Ignore rsync default configuration'
      ssh_api_RSYNC_OPTS_ARRAY_add_opts "${xoptions[@]}" || return $?
    else
      if [ "${RSYNC_NOW_API_SKIP_COMPRESS:-}" = 'true' ] ; then
        ssh_api_RSYNC_OPTS_ARRAY_add_opts_from_string "${RSYNC_OPTS_LARGE_FILES}" || return $?
      else
        ssh_api_RSYNC_OPTS_ARRAY_add_opts_from_string "${RSYNC_OPTS_QUICK}" || return $?
        ssh_api_RSYNC_OPTS_ARRAY_add_opts_from_string "--skip-compress=${RSYNC_SKIP_COMPRESS_DEFAULTS}" || return $?
      fi
      ssh_api_RSYNC_OPTS_ARRAY_add_opts "${xoptions[@]}" || return $?
    fi

    if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
      echodebug "${FUNCNAME[0]}()"
      ssh_api_RSYNC_OPTS_ARRAY_show_opts
    fi

    run_rsync_now "${source_folder}" "${_ssh_user_}" "${_ssh_address_}" "${target_folder}"
  } || return $?
}

function run_rsync_now {
  local -r _local_folder_="$1"
  local -r _ssh_user_="$2"
  local -r _ssh_host_="$3"
  local -r _target_folder_="$4"

  exit_if_directory_not_exists "${_local_folder_}"
  exit_if_variable_not_define_or_empty '_target_folder_'

  if [ -n "${_ssh_user_}" ] ; then
    exit_if_variable_not_define_or_empty '_ssh_host_'
    echodebug "ssh_api_rsync_directory_to_host '${_local_folder_}' '${_ssh_user_}' '${_ssh_host_}' '${_target_folder_}'"

    ssh_api_rsync_directory_to_host "${_local_folder_}" "${_ssh_user_}" "${_ssh_host_}" "${_target_folder_}"
  else
    if [ -n "${_ssh_host_}" ] ; then
      echowarn "ssh_address is define ('${_ssh_host_}') but will be ignored (no ssh_user)" >&2
    fi

    ssh_api_rsync_directory_locally "${_local_folder_}" "${_target_folder_}"
  fi
}

