#!/bin/bash
#
# rsync-now:bin/apis/rsync-now-config-api.sh
#

function get_configuration_json_or_create_if_needed {
  if [ ! -d "${_RSYNC_NOW_CONFIG_HOME_}" ] ; then
    mkdir -p "${_RSYNC_NOW_CONFIG_HOME_}" >&2 || return $?
  fi

  local json_file=
  json_file="$( get_configuration_json_filename )" || return $?

  if [ -f "${json_file}" ] ; then
    cat "${json_file}" || return $?
  else
    echowarn "No configuration file found. Create '${json_file}'"
    echo '{ "declare": {"ssh_user":"","ssh_address":""}, "sync": [] }' | jq -c '.' || return $?
  fi
}

function get_configuration_names_list {
  find "${_RSYNC_NOW_CONFIG_HOME_}" -maxdepth 1 -type f -name '*.json' -exec basename {} \; |
    sed 's,.json$,,g' || return $?
}

function get_index_for_local_dir {
  local -r json_config="$1"
  local -r local_dir="$2"

  local json_entries=
  json_entries="$( get_rsync_configs "${json_config}" )" || return $?
  local -r json_entries

  local -i index=0
  local -i selected_index=-1
  local duplicate_found='false'
  local json=
  local id=

  while read -r json ; do
    if [ -n "${json}" ] ; then
      id="$( get_value_from_json ".local" "${json}" )" || return $?
      if [ "${id}" = "${local_dir}" ] ; then
        if [ "${selected_index}" -ge 0 ] ; then
          echowarn "Duplicate found for '${id}'"
          echo "${json}" >&2
          duplicate_found='true'
        else
          selected_index="${index}"
        fi
      fi
      ((index++))
    fi
  done <<<"${json_entries}"

  if [ "${duplicate_found}" = 'true' ] ; then
    return 100
  fi

  if [ "${selected_index}" -ge 0 ] ; then
    echo "${selected_index}"
  fi
}

function get_json_by_index {
  local -ri index="$1"

  get_configuration_json_if_exists | jq ".sync[${index}]" || return $?
}

function get_json_by_local_dir {
  local -r json_config="$1"
  local -r local_dir="$2"

  local json_entries=
  json_entries="$( get_rsync_configs "${json_config}" )" || return $?
  local -r json_entries

  local json=
  local selected_json=
  local duplicate_found='false'
  local id=

  while read -r json ; do
    if [ -n "${json}" ] ; then
      id="$( get_value_from_json ".local" "${json}" )" || return $?
      if [ "${id}" = "${local_dir}" ] ; then
        if [ -n "${selected_json}" ] ; then
          echowarn "Duplicate found for '${id}'"
          echo "${json}" >&2
          duplicate_found='true'
        else
          selected_json="${json}"
        fi
      fi
    fi
  done <<<"${json_entries}"

  if [ "${duplicate_found}" = 'true' ] ; then
    return 100
  fi

  if [ -n "${selected_json}" ] ; then
    echo "${selected_json}"
  fi
}

function rsync_now_config_delete_entry {
  local -r local_dir="$1"

  #
  # (re)Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  local index=
  index="$( get_index_for_local_dir "${json_config}" "${local_dir}" )" || return $?

  if [ -n "${index}" ] ; then
    local json=
    json="$( get_configuration_json_or_create_if_needed )" || return $?

    local config_file=
    config_file="$( get_configuration_json_filename )" || return $?

    jq "del( .sync[${index}] )" <<<"${json}" >"${config_file}.new" || return $?
    mv -f "${config_file}.new" "${config_file}" || return $?
  else
    echoerror "Entry '${local_dir}' not found."
    return 100
  fi
}

function rsync_now_config_delete_variable {
  local -r varname="$1"

  local json=
  json="$( get_configuration_json_if_exists )" || return $?

  local config_file=
  config_file="$( get_configuration_json_filename )" || return $?

  jq "del( .declare.${varname} )" <<<"${json}" >"${config_file}.new" || return $?
  mv -f "${config_file}.new" "${config_file}" || return $?
}

function rsync_now_config_list_entries {
  #
  # (re)Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  echoinfo 'List of local dirs'
  get_rsync_configs "${json_config}" | jq -r '.local' || return $?
}

function rsync_now_config_list_variables {
  #
  # (re)Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  echoinfo 'List of variables'
  get_vars_config "${json_config}" | jq -r 'keys[]' || return $?
}

function rsync_now_config_new_entry {
  local -r local_dir="$1"
  local -r remote_dir="$2"
  local -r description="$3"

  #
  # (re)Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_or_create_if_needed )" || return $?
  local -r json_config

  local index=
  index="$( get_index_for_local_dir "${json_config}" "${local_dir}" )" || return $?

  if [ -n "${index}" ] ; then
    echoerror "Entry '${local_dir}' already exists."
    return 10
  else
    local config_file=
    config_file="$( get_configuration_json_filename )" || return $?

    (
      [ "${DEBUG:-}" = 'true' ] && set -x
      jq <<<"${json_config}" >"${config_file}.new" \
        --arg 'DESCRIPTION' "${description}" \
        --arg 'LOCALDIR'    "${local_dir}" \
        --arg 'REMOTEDIR'   "${remote_dir}" \
        '.sync[ .sync | length ] |= . +
{
  "description": $DESCRIPTION,
  "enabled": "on",
  "local": $LOCALDIR,
  "options": {
    "exclude": [],
    "no-default": false,
    "raw": []
  },
  "remote": $REMOTEDIR
}'
    ) || return $?
    mv -f "${config_file}.new" "${config_file}" || return $?
  fi
}

function rsync_now_config_set_entry_property {
  local -r local_dir="$1"
  local -r entry_name="$2"
  local -r entry_value="$3"
  local -r entry_value_type="$4"

  case "${entry_value_type}" in
    'string')
      local -r entry_value_is_json='false'
      ;;

    'json')
      local -r entry_value_is_json='true'
      ;;

    *)
      echofatal "${FUNCNAME[0]}() - Internal error, wrong entry_value_type='${entry_value_type}'"
      return 100
      ;;
  esac

  #
  # (re)Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  local index=
  index="$( get_index_for_local_dir "${json_config}" "${local_dir}" )" || return $?

  if [ -n "${index}" ] ; then
    local json_for_entry=
    json_for_entry="$( get_json_by_index "${index}" )" || return $?

    if [ "${entry_value_is_json}" = 'true' ] ; then
      json_for_entry="$(
        jq -c <<<"${json_for_entry}" \
            --arg 'name' "${entry_name}" \
            --argjson 'value' "${entry_value}" \
            '. += ( [{"key":$name,"value":$value}] | from_entries )'
        )" || return $?
    else
      json_for_entry="$(
        jq -c <<<"${json_for_entry}" \
            --arg 'name' "${entry_name}" \
            --arg 'value' "${entry_value}" \
            '. += ( [{"key":$name,"value":$value}] | from_entries )'
        )" || return $?
    fi

    local json=
    json="$( get_configuration_json_or_create_if_needed )" || return $?

    local config_file=
    config_file="$( get_configuration_json_filename )" || return $?

    jq ".sync[${index}] = ${json_for_entry}" <<<"${json}" >"${config_file}.new" || return $?
    mv -f "${config_file}.new" "${config_file}" || return $?
  else
    echoerror "Entry '${local_dir}' not found."
    return 100
  fi
}

function rsync_now_config_set_or_and_variable {
  local -r varname="$1"
  local -r varvalue="$2"

  local config_file=
  config_file="$( get_configuration_json_filename )" || return $?

  local json=
  json="$( get_configuration_json_or_create_if_needed )" || return $?

  (
    [ "${DEBUG:-}" = 'true' ] && set -x
    jq <<<"${json}" >"${config_file}.new" \
        --arg 'name' "${varname}" \
        --arg 'value' "${varvalue}" \
        '.declare += ( [{"key":$name,"value":$value}] | from_entries )'
  ) || return $?
  mv -f "${config_file}.new" "${config_file}" || return $?
}

function rsync_now_config_show_entry {
  local -r local_dir="$1"

  #
  # (re)Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  local json=
  json="$( get_json_by_local_dir "${json_config}" "${local_dir}" )" || return $?

  if [ -n "${json}" ] ; then
    echo "${local_dir}"
    get_value_from_json '.remote' "${json}" || return $?
    get_value_from_json '.description' "${json}" || return $?
    get_value_from_json '.enabled' "${json}" || return $?
    get_options_json_config "${json}" || return $?
  else
    echoerror "Can't find entry for: '${local_dir}'."
    return 100
  fi
}

function rsync_now_config_show_filename {
  get_configuration_json_filename || return $?
}

function rsync_now_config_show_variable {
  local -r varname="$1"

  if ! get_value_from_json ".declare.${varname}" "$( get_configuration_json_if_exists )" ; then
    echoerror "Can't find '${varname}'"
    exit 100
  fi
}
