#!/bin/bash
#
# rsync-now:bin/rsync-now-config.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/apis/rsync-now-common-config-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/apis/rsync-now-config-api.sh" || exit $?

function usage {
  cat <<EOF
Configuration tool for rsync-now

Usage:

   $( basename "$0" ) --help

   $( basename "$0" ) --new LOCAL_DIR REMOTE_DIR DESCRIPTION [--config-name CONF_NAME]

   $( basename "$0" ) --set-remote LOCAL_DIR REMOTE_DIR [--config-name CONF_NAME]
   $( basename "$0" ) --set-description LOCAL_DIR DESCRIPTION [--config-name CONF_NAME]
   $( basename "$0" ) --set-enabled LOCAL_DIR on|off [--config-name CONF_NAME]
   $( basename "$0" ) --set-options-json LOCAL_DIR RSYNC_OPTIONS [--config-name CONF_NAME]

   $( basename "$0" ) --delete LOCAL_DIR [--config-name CONF_NAME]

   $( basename "$0" ) --declare-ssh-user SSH_USER [--config-name CONF_NAME]
   $( basename "$0" ) --declare-ssh-address SSH_ADDRESS [--config-name CONF_NAME]
   $( basename "$0" ) --declare VARNAME VALUE [--config-name CONF_NAME]

   $( basename "$0" ) --delete-variable VARNAME [--config-name CONF_NAME]

   $( basename "$0" ) --list
   $( basename "$0" ) --list-entries [--config-name CONF_NAME]
   $( basename "$0" ) --list-variables [--config-name CONF_NAME]
   $( basename "$0" ) --show-entry LOCAL_DIR [--config-name CONF_NAME]
   $( basename "$0" ) --show-filename [--config-name CONF_NAME]
   $( basename "$0" ) --show-variable VARNAME [--config-name CONF_NAME]

Actions:

  -h,       --help  Displays help message and exits.

            --new LOCAL_DIR REMOTE_DIR DESCRIPTION
                        Add a new entry.

            --set-remote LOCAL_DIR REMOTE_DIR
                        Updates path of remote folder (backup path) for the LOCAL_DIR entry.
            --set-description LOCAL_DIR DESCRIPTION
                        Updates description for the LOCAL_DIR entry.
            --set-enabled LOCAL_DIR on|off
                        Updates default activation state for the LOCAL_DIR entry.
            --set-options-json LOCAL_DIR RSYNC_OPTIONS
                        Set specific options for rsync for the LOCAL_DIR entry using JSON format.
                        Typically the string '{ "exclude": [] , "raw": [] }' is exatly the same has
                        no configuration.
                        You can define directories or files to exclude like this:
                        '{ "exclude": [ "dir1/to/ignore", "dir/to/ignore/number2" ] , "raw": [] }'
                        You can also add any valid rsync options using "raw" array:
                        '{ "exclude": [ "dir/to/ignore" ] ,"raw": [ "--no-links" ] }'

            --delete LOCAL_DIR
                        Delete all the configuration related to LOCAL_DIR.

            --declare-ssh-user SSH_USER
                        Configure user name the ssh protocol (common setting).
            --declare-ssh-address SSH_ADDRESS
                        Configure the ssh address of the remote server (common setting).
            --declare VARNAME VALUE
                        Add or modify a variable defined by its name and its value.

            --delete-variable VARNAME
                        Remove the variable from its name.
            --list
                        Display available configuration names, list of values available for
                        option --config-name.
            --list-entries
                        Displays the names of all entries (local directory).
            --list-variables
                        Display all defined variable names.

            --show-entry LOCAL_DIR
                        Displays all values for LOCAL_DIR entry.
            --show-filename
                        Display configuration file name and exit.
            --show-variable VARNAME
                        Displays the value of VARNAME.

Parameters:

  -C CONF_NAME, --config-name CONF_NAME
                        Defines a namespace for the configuration.
  -V,           --verbose
                        More verbose output (debug mode)

EOF
}

function rsync_now_config_list_configs {
  get_configuration_names_list || return $?
}

function P2_param {
  if [ "$#" -gt 0 ] ; then
    echo "$1"
  else
    echofatal "2 values are required for '${param}'"
    usage >&2
    return 1
  fi
}

function P3_param {
  if [ "$#" -gt 0 ] ; then
    echo "$1"
  else
    echofatal "3 values are required for '${param}'"
    usage >&2
    return 1
  fi
}

function main {
  local param=
  local param_description=
  local param_local_dir=
  local param_name=
  local param_on_off=
  local param_options=
  local param_remote_dir=
  local param_rsync_custom_json_config_name=
  local param_value=
  local action=

  if [ "$#" -eq 0 ] ; then
    echofatal 'Parameters missing.'
    usage >&2
    exit 1
  fi

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        return 0
        ;;

      '-C'|'--config-name')
        # shellcheck disable=SC2034
        param_rsync_custom_json_config_name="$( P_param "$@" )" && shift || return $?
        ;;

      '--new') # LOCAL_DIR REMOTE_DIR DESCRIPTION
        action='--new'
        param_local_dir="$( P3_param "$@" )" && shift || return $?
        param_remote_dir="$( P3_param "$@" )" && shift || return $?
        param_description="$( P3_param "$@" )" && shift || return $?
        ;;

      '--set-remote') # LOCAL_DIR REMOTE_DIR
        action='--set-remote'
        param_local_dir="$( P2_param "$@" )" && shift || return $?
        param_remote_dir="$( P2_param "$@" )" && shift || return $?
        ;;

      '--set-description') # LOCAL_DIR DESCRIPTION
        action='--set-description'
        param_local_dir="$( P2_param "$@" )" && shift || return $?
        param_description="$( P2_param "$@" )" && shift || return $?
        ;;

      '--set-enabled') # LOCAL_DIR on|off
        action='--set-enabled'
        param_local_dir="$( P2_param "$@" )" && shift || return $?
        param_on_off="$( P2_param "$@" )" && shift || return $?

        # TODO check param_on_off
        ;;

      '--set-options-json') # LOCAL_DIR OPTIONS
        action='--set-options-json'
        param_local_dir="$( P2_param "$@" )" && shift || return $?
        param_options="$( P2_param "$@" )" && shift || return $?
        ;;

      '--delete') # LOCAL_DIR
        action='--delete'
        param_local_dir="$( P_param "$@" )" && shift || return $?
        ;;

      '--declare-ssh-user') # SSH_USER
        action='--declare-ssh-user'
        param_value="$( P_param "$@" )" && shift || return $?
        ;;

      '--declare-ssh-address') # SSH_ADDRESS
        action='--declare-ssh-address'
        param_value="$( P_param "$@" )" && shift || return $?
        ;;

      '--declare') # VARNAME VALUE
        action='--declare'
        param_name="$( P2_param "$@" )" && shift || return $?
        param_value="$( P2_param "$@" )" && shift || return $?
        ;;

      '--delete-variable') # VARNAME
        action='--delete-variable'
        param_name="$( P_param "$@" )" && shift || return $?
        ;;

      '--list')
        action='--list'
        ;;

      '--list-entries')
        action='--list-entries'
        ;;

      '--list-variables')
        action='--list-variables'
        ;;

      '--show-entry') # LOCAL_DIR
        action='--show-entry'
        param_local_dir="$( P_param "$@" )" && shift || return $?
        ;;

      '--show-filename')
        action='--show-filename'
        ;;

      '--show-variable') # VARNAME
        action='--show-variable'
        param_name="$( P_param "$@" )" && shift || return $?
        ;;

      '-V'|'--verbose')
        declare -gx DEBUG='true'
        declare -gx MESSAGE_API_SHOW_DEBUG='true'
        declare -gx SSH_API_RSYNC_DEBUG='true'
        ;;

      *)
        echofatal "Unknown option: '${param}'."
        usage >&2
        return 1
        ;;
    esac
  done

  if [ -z "${action}" ] ; then
    echofatal 'Action is missing'
    usage >&2
    return 1
  else
    case "${action}" in
      '--new') # LOCAL_DIR REMOTE_DIR DESCRIPTION
        rsync_now_config_new_entry "${param_local_dir}" "${param_remote_dir}" "${param_description}" || return $?
        ;;

      '--set-remote') # LOCAL_DIR REMOTE_DIR
        rsync_now_config_set_entry_property "${param_local_dir}" 'remote' "${param_remote_dir}" 'string' || return $?
        ;;

      '--set-description') # LOCAL_DIR DESCRIPTION
        rsync_now_config_set_entry_property "${param_local_dir}" 'description' "${param_description}" 'string' || return $?
        ;;

      '--set-enabled') # LOCAL_DIR on|off
        rsync_now_config_set_entry_property "${param_local_dir}" 'enabled' "${param_on_off}" 'string' || return $?
        ;;

      '--set-options-json') # LOCAL_DIR OPTIONS
        rsync_now_config_set_entry_property "${param_local_dir}" 'options' "${param_options}" 'json' || return $?
        ;;

      '--delete') # LOCAL_DIR
        rsync_now_config_delete_entry "${param_local_dir}" || return $?
        ;;

      '--declare-ssh-user') # SSH_USER
        rsync_now_config_set_or_and_variable 'ssh_user' "${param_value}" || return $?
        ;;

      '--declare-ssh-address') # SSH_ADDRESS
        rsync_now_config_set_or_and_variable 'ssh_address' "${param_value}" || return $?
        ;;

      '--declare') # VARNAME VALUE
        rsync_now_config_set_or_and_variable "${param_name}" "${param_value}" || return $?
        ;;

      '--delete-variable') # VARNAME
        rsync_now_config_delete_variable "${param_name}" || return $?
        ;;

      '--list')
        rsync_now_config_list_configs || return $?
        ;;

      '--list-entries')
        rsync_now_config_list_entries || return $?
        ;;

      '--list-variables')
        rsync_now_config_list_variables || return $?
        ;;

      '--show-entry') # LOCAL_DIR
        rsync_now_config_show_entry "${param_local_dir}" || return $?
        ;;

      '--show-filename')
        rsync_now_config_show_filename || return $?
        ;;

      '--show-variable') # VARNAME
        rsync_now_config_show_variable "${param_name}" || return $?
        ;;

      *)
        echofatal "Unknown option: '${param}'."
        usage >&2
        return 1
        ;;
    esac
  fi
}

main "$@" || exit $?
