#!/bin/bash
#
# rsync-now:bin/rsync-now-purge-remotes.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/apis/rsync-now-common-config-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/apis/utils-api.sh" || exit $?

function usage {
  cat <<EOF
Clean remore (or destination) directory that should be exclude be rsync-now

Usage:
  $( basename "$0" ) --help
  $( basename "$0" ) [--config-name CONF_NAME]

Options:
  -h,           --help  Displays help message and exits.
  -C CONF_NAME, --config-name CONF_NAME
                        Defines namespace for the configuration.
  -V,           --verbose
                        More verbose output (debug mode)
  -D,           --dryrun
                        Don't delete file, just pretend doing it

EOF
}

function test_remote_directory_exists {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_remote_dir_="$3"

  if [ -z "${_ssh_host_}" ] ; then
    test -d "${_ssh_remote_dir_}"
 elif [ "$( ssh_api_test_remote_directory "${_ssh_user_}" "${_ssh_host_}" "${_ssh_remote_dir_}" )" = 'FOUND' ] ; then
    return 0
 else
    return 1
 fi
}

function __cleanning_remote__ {
  local -r _json_="$1"
  local -r _ssh_user_="$2"
  local -r _ssh_address_="$3"

  local remote=
  remote="$( get_substituted_value_from_json '.remote' "${_json_}" )" || return $?
  local -r remote

  if [ -z "${remote}" ] ; then
    echoerror 'Remote field is empty (skipping)'
    return 0
  fi
  echoinfo "Discover remote (resolved): '${remote}'"

  if ! test_remote_directory_exists "${_ssh_user_}" "${_ssh_address_}" "${remote}" ; then
    echoerror 'Remote path not found (skipping)'
    return 0
  fi

  local _options_json_=
  _options_json_="$( get_substituted_options_json_config "${_json_}" )" || return $?

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug 'Options'
    jq '.' <<<"${_options_json_}" >&2
  fi

  local exclude_entries=
  exclude_entries="$( get_options_excludes_from_json "${_options_json_}" )" || return $?
  local -r exclude_entries

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug '--- exclude_entries: --['
    echodebug_ml "${exclude_entries}"
    echodebug ']-- exclude_entries: ---'
  fi

  pushd "${remote}" >/dev/null || return $?
  echoinfo "PWD: $( pwd )"

  local _dir_=
  while read -r _dir_ ; do
    echoinfo "Handle: ${_dir_}"
    if [ -e "${_dir_}" ] && [ "${param_dryrun}" != 'true' ]; then
      rm -frv "${_dir_}" || return $?
    fi
  done <<<"${exclude_entries}"
  popd >/dev/null || return $?
}

function cleanning_remotes {
  local -r json_config="$1"
  local -r _ssh_user_="$2"
  local -r _ssh_address_="$3"

  local sync_json=
  sync_json="$( get_rsync_configs "${json_config}" )" || return $?
  local -r sync_json

  local entry=
  while read -r entry ; do
    if [ -n "${entry}" ] ; then
      __cleanning_remote__ "${entry}" "${_ssh_user_}" "${_ssh_address_}" || return $?
    fi
  done <<<"${sync_json}"
}

function main {
  local param=
  local param_dryrun=
  local param_rsync_custom_json_config_name=BACKUP # FIXME File is ~/.config/rsync-now/BACKUP.json

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-D'|'--dryrun')
        param_dryrun='true'
        ;;

      '-C'|'--config-name')
        # shellcheck disable=SC2034
        param_rsync_custom_json_config_name="$( P_param "$@" )" && shift || exit $?
        ;;

      '-V'|'--verbose')
        declare -grx SSH_API_RSYNC_DEBUG='true'
        declare -grx MESSAGE_API_SHOW_DEBUG='true'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  #
  # Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  #
  # Read and initialiaze variables
  #
  rsync_now_load_vars "${json_config}" || return $?

  if ! is_variable_define_safe 'ssh_user' ; then
    echofatal "Can't get .ssh.user"
    exit 100
  fi
  # shellcheck disable=SC2154
  local -r _ssh_user_="${ssh_user}"

  if ! is_variable_define_safe 'ssh_address' ; then
    echofatal "Can't get .ssh.address"
    exit 100
  fi
  # shellcheck disable=SC2154
  local -r _ssh_address_="${ssh_address}"

  cleanning_remotes "${json_config}" "${_ssh_user_}" "${_ssh_address_}" || return $?

  echookay "$( basename "${BASH_SOURCE[0]}" ) - Done."
}

main "$@" || exit $?
