#!/bin/bash
#
# rsync-now:bin/rsync-now.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/ssh-api/rsync-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/apis/rsync-now-common-config-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/apis/rsync-now-api.sh" || exit $?

function usage {
  cat <<EOF
rsync wrapper for synchronizing preconfigured folders.

Usage:
  $( basename "$0" ) --help
  $( basename "$0" ) [--all|--enabled] [--config-name CONF_NAME] [--verbose] [--checksum]

Options:
  -h,           --help  Displays help message and exits.
  -A,           --all   Synchronize all folders for current configuration (non interactive).
  -C CONF_NAME, --config-name CONF_NAME
                        Defines namespace for the configuration.
  -D,           --default
                        Synonym for --enable
  -E,           --enabled
                        Synchronize enabled folders for current configuration (non interactive).
  -I,           --ignore-destination-checks
                        Allow to run backup even if destination directory does not exists.
  -V,           --verbose
                        More verbose output (debug mode)
                --checksum
                        Don't rely on modification time, use checksum instead.
                --no-compress
                        Don't use compress
                --debug
                        Alias for --verbose
EOF
}

function main {
  local param=
  local param_mode='interactive'
  local param_rsync_custom_json_config_name=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-C'|'--config-name')
        # shellcheck disable=SC2034
        param_rsync_custom_json_config_name="$( P_param "$@" )" && shift || exit $?
        ;;

      '-A'|'--all')
        param_mode='all'
        ;;

      '-E'|'--enabled'|'-D'|'--default')
        param_mode='enabled'
        ;;

      '-I'|'--ignore-destination-checks')
        declare -gx RSYNC_CHECK_DESTINATION='false'
        ;;

      '--checksum')
        declare -gx RSYNC_NOW_API_CHECKSUM='true'
        ;;

      '--no-compress')
        declare -gx RSYNC_NOW_API_SKIP_COMPRESS='true'
        ;;

      '-V'|'--verbose'|'--debug')
        declare -gx SSH_API_RSYNC_DEBUG='true'
        declare -gx MESSAGE_API_SHOW_DEBUG='true'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        return 1
        ;;
    esac
  done

  #
  # Load configuration
  #
  local json_config=
  json_config="$( get_configuration_json_if_exists )" || return $?
  local -r json_config

  #
  # Read and initialiaze variables
  #
  rsync_now_load_vars "${json_config}" || return $?

  if ! is_variable_define_safe 'ssh_user' ; then
    echofatal "Can't get .ssh.user"
    return 100
  fi
  # shellcheck disable=SC2154
  local -r _ssh_user_="${ssh_user}"

  if ! is_variable_define_safe 'ssh_address' ; then
    echofatal "Can't get .ssh.address"
    return 100
  fi
  # shellcheck disable=SC2154
  local -r _ssh_address_="${ssh_address}"

  build_gconfig "${json_config}" || return $?

  case "${param_mode}" in
    'all')
      rsync_now_for_all_ids "${_ssh_user_}" "${_ssh_address_}" || return $?
      ;;

    'enabled')
      rsync_now_for_enabled_ids "${_ssh_user_}" "${_ssh_address_}" || return $?
      ;;

    'interactive')
      rsync_now_display_choices "${_ssh_user_}" "${_ssh_address_}" || return $?
      ;;

    *)
      echofatal "Internal error: param_mode='${param_mode}'"
      return 1
      ;;
  esac
}

main "$@" || exit $?
